@@ -278,6 +278,8 @@ public:
   void handlePhysicalInfo(ArNetPacket *packet);
   /// This callback is called when an update on the temperature information changes
   void handleTemperatureInfo(ArNetPacket *packet);
+  /// Called when the map on the server changes.
+  void handleMapUpdated(ArNetPacket *packet);
 protected:
 
   /// The results from the data update are stored in these variables
@@ -306,6 +308,7 @@ protected:
   ArFunctor1C<OutputHandler, ArNetPacket *> myHandleBatteryInfoCB;
   ArFunctor1C<OutputHandler, ArNetPacket *> myHandlePhysicalInfoCB;
   ArFunctor1C<OutputHandler, ArNetPacket *> myHandleTemperatureInfoCB;
+  ArFunctor1C<OutputHandler, ArNetPacket *> myHandleMapUpdatedCB;
   //@}
   
   /// A header for the columns in the data printout is sometimes printed
@@ -322,6 +325,7 @@ OutputHandler::OutputHandler(ArClientBase *client) :
   myHandleBatteryInfoCB(this, &OutputHandler::handleBatteryInfo),
   myHandlePhysicalInfoCB(this, &OutputHandler::handlePhysicalInfo),
   myHandleTemperatureInfoCB(this, &OutputHandler::handleTemperatureInfo),
+  myHandleMapUpdatedCB(this, &OutputHandler::handleMapUpdated),
   myNeedToPrintHeader(false),
   myGotBatteryInfo(false)
 {
@@ -363,6 +367,12 @@ OutputHandler::OutputHandler(ArClientBase *client) :
     myClient->addHandler("update", &myHandleOutputCB);
     myClient->request("update", 100);
   }
+
+  if(myClient->dataExists("mapUpdated"))
+  {
+    myClient->addHandler("mapUpdated", &myHandleMapUpdatedCB);
+    myClient->request("mapUpdated", -1);
+  }
 }
 
 OutputHandler::~OutputHandler(void)
@@ -500,6 +510,13 @@ void OutputHandler::handleTemperatureInfo(ArNetPacket *packet)
   myNeedToPrintHeader = true;
 }
 
+void OutputHandler::handleMapUpdated(ArNetPacket *packet)
+{
+  printf("\nMap changed.\n");
+  fflush(stdout);
+  myNeedToPrintHeader = true;
+}
+
 
 /* Key handler for the escape key: shutdown all of Aria. */
 void escape(void)
