@@ -1,8 +1,8 @@
 /*
-MobileRobots Advanced Robotics Interface for Applications (ARIA)
+Adept MobileRobots Robotics Interface for Applications (ARIA)
 Copyright (C) 2004, 2005 ActivMedia Robotics LLC
 Copyright (C) 2006, 2007, 2008, 2009, 2010 MobileRobots Inc.
-Copyright (C) 2011, 2012 Adept Technology
+Copyright (C) 2011, 2012, 2013 Adept Technology
 
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -19,9 +19,9 @@ Copyright (C) 2011, 2012 Adept Technology
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 If you wish to redistribute ARIA under different terms, contact 
-MobileRobots for information about a commercial version of ARIA at 
+Adept MobileRobots for information about a commercial version of ARIA at 
 robots@mobilerobots.com or 
-MobileRobots Inc, 10 Columbia Drive, Amherst, NH 03031; 800-639-9481
+Adept MobileRobots, 10 Columbia Drive, Amherst, NH 03031; +1-603-881-7960
 */
 #ifndef ARACTIONDESIRED_H
 #define ARACTIONDESIRED_H
@@ -230,9 +230,12 @@ public:
   AREXPORT static const double MAX_STRENGTH;
   /// Constructor
   ArActionDesired() 
-    { myHeadingSet = false; 
-    myTransDecelDes.setOverrideDoesLessThan(false); 
-    myRotDecelDes.setOverrideDoesLessThan(false); }
+    { 
+      myHeadingSet = false; 
+      myTransDecelDes.setOverrideDoesLessThan(false); 
+      myRotDecelDes.setOverrideDoesLessThan(false);
+      myMaxNegVelDes.setOverrideDoesLessThan(false); 
+    }
   /// Destructor
   virtual ~ArActionDesired() {}
   
@@ -841,6 +844,9 @@ public:
   AREXPORT virtual void log(void) const;
   /// Gets whether anything is desired (should only really be used in relation to logging)
   AREXPORT virtual bool isAnythingDesired(void) const;
+  /// Do a sanity check on the action (this is set up this way so the
+  /// action name can be passed in)
+  AREXPORT virtual void sanityCheck(const char *actionName);
 
 protected:
   double myHeading;
