@@ -1,8 +1,8 @@
 /*
-MobileRobots Advanced Robotics Interface for Applications (ARIA)
+Adept MobileRobots Robotics Interface for Applications (ARIA)
 Copyright (C) 2004, 2005 ActivMedia Robotics LLC
 Copyright (C) 2006, 2007, 2008, 2009, 2010 MobileRobots Inc.
-Copyright (C) 2011, 2012 Adept Technology
+Copyright (C) 2011, 2012, 2013 Adept Technology
 
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -19,9 +19,9 @@ Copyright (C) 2011, 2012 Adept Technology
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 If you wish to redistribute ARIA under different terms, contact 
-MobileRobots for information about a commercial version of ARIA at 
+Adept MobileRobots for information about a commercial version of ARIA at 
 robots@mobilerobots.com or 
-MobileRobots Inc, 10 Columbia Drive, Amherst, NH 03031; 800-639-9481
+Adept MobileRobots, 10 Columbia Drive, Amherst, NH 03031; +1-603-881-7960
 */
 
 #ifndef ARSOUNDSQUEUE_H
@@ -56,6 +56,7 @@ MobileRobots Inc, 10 Columbia Drive, Amherst, NH 03031; 800-639-9481
  * @sa ArSoundPlayer
  * @sa @ref soundsQueueExample.cpp
  *
+  @ingroup UtilityClasses
  */ 
 
 class ArSoundsQueue: public ArASyncTask
@@ -261,7 +262,7 @@ public:
   }
 
   /** Add a callback functor to be invoked when plackback of one sound or speech
-   * utterance begins. */
+   * utterance finishes */
   AREXPORT void addSoundFinishedCallback(ArFunctor* f)
   {
     myEndPlaybackCBList.push_back(f);
@@ -274,6 +275,36 @@ public:
     myEndPlaybackCBList.remove(f);
   }
 
+  /** Add a callback functor to be invoked when playback of one sound or speech utterance starts. */
+  AREXPORT void addSoundItemStartedCallback(
+	  ArFunctor1<ArSoundsQueue::Item> *f)
+  {
+    myStartItemPlaybackCBList.push_back(f);
+  }
+
+  /** Remove a callback functor to be invoked when playback one sound or speech utterance starts. */
+  AREXPORT void remSoundItemStartedCallback(
+	  ArFunctor1<ArSoundsQueue::Item> *f)
+  {
+    myStartItemPlaybackCBList.remove(f);
+  }
+
+  /** Add a callback functor to be invoked when plackback of one sound or speech
+   * utterance finishes */
+  AREXPORT void addSoundItemFinishedCallback(
+	  ArFunctor1<ArSoundsQueue::Item> *f)
+  {
+    myEndItemPlaybackCBList.push_back(f);
+  }
+
+  /** Remove a callback functor to be invoked when plackback of one sound or
+   * speech utterance finishes. */
+  AREXPORT void remSoundItemFinishedCallback(
+	  ArFunctor1<ArSoundsQueue::Item> *f)
+  {
+    myEndItemPlaybackCBList.remove(f);
+  }
+
   /** Add a callback functor to be invoked when a the sound queue becomes
    * non-empty, that is, when a block of sounds/speech utterances begins.
    */
@@ -332,6 +363,12 @@ public:
   /** Remove pending items with the given type. */
   AREXPORT void removePendingItems(ItemType type);
 
+  /** Remove pending and current items with a priority less than that given. */
+  AREXPORT void removeItems(int priority);
+
+  /** Removes pending and current items with the same data and type as the given item **/
+  AREXPORT void removeItems(Item item);
+
   AREXPORT std::string nextItem(ItemType type);
   AREXPORT std::string nextItem(int priority);
   AREXPORT std::string nextItem(ItemType type, int priority);
@@ -442,7 +479,7 @@ public:
   //@}
 
   /** Set a playback condition functor used for default speech and sound file
-   * items */
+   * items. Set to NULL to clear. */
   AREXPORT void setDefaultPlayConditionCB(PlaybackConditionFunctor* f) {
     myDefaultPlayConditionCB = f;
   }
@@ -486,6 +523,8 @@ protected:
   //@{
   std::list<ArFunctor*> myStartPlaybackCBList;
   std::list<ArFunctor*> myEndPlaybackCBList;
+  std::list<ArFunctor1<ArSoundsQueue::Item> *> myStartItemPlaybackCBList;
+  std::list<ArFunctor1<ArSoundsQueue::Item> *> myEndItemPlaybackCBList;
   std::list<ArFunctor*> myQueueNonemptyCallbacks;
   std::list<ArFunctor*> myQueueEmptyCallbacks;
   //@}
