@@ -1,8 +1,8 @@
 /*
-MobileRobots Advanced Robotics Interface for Applications (ARIA)
+Adept MobileRobots Robotics Interface for Applications (ARIA)
 Copyright (C) 2004, 2005 ActivMedia Robotics LLC
 Copyright (C) 2006, 2007, 2008, 2009, 2010 MobileRobots Inc.
-Copyright (C) 2011, 2012 Adept Technology
+Copyright (C) 2011, 2012, 2013 Adept Technology
 
      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
@@ -19,9 +19,9 @@ Copyright (C) 2011, 2012 Adept Technology
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 If you wish to redistribute ARIA under different terms, contact 
-MobileRobots for information about a commercial version of ARIA at 
+Adept MobileRobots for information about a commercial version of ARIA at 
 robots@mobilerobots.com or 
-MobileRobots Inc, 10 Columbia Drive, Amherst, NH 03031; 800-639-9481
+Adept MobileRobots, 10 Columbia Drive, Amherst, NH 03031; +1-603-881-7960
 */
 #include "ArExport.h"
 #include "ariaOSDef.h"
@@ -115,6 +115,7 @@ AREXPORT void ArSocket::shutdown()
 {
   if (ourInitialized)
   {
+    ArLog::log(ArLog::Verbose, "ArSocket::shutdown calling WSACleanup");
     WSACleanup();
     ourInitialized=false;
   }
@@ -524,14 +525,14 @@ AREXPORT bool ArSocket::close()
 
   if (myFD != INVALID_SOCKET) {
     if (!myHost.empty() && (myPort != -1)) {
-      ArLog::log(ArLog::Verbose, "Closing %s socket %s:%i (0x%x)",
+      ArLog::log(ArLog::Verbose, "Closing %s socket %s:%i (0x%p)",
                  toString(myType),
                  myHost.c_str(),
                  myPort,
                  myFD);
     }
     else {
-      ArLog::log(ArLog::Verbose, "Closing %s socket (0x%x)",
+      ArLog::log(ArLog::Verbose, "Closing %s socket (0x%p)",
                  toString(myType),
                  myFD);
     }
@@ -545,7 +546,7 @@ AREXPORT bool ArSocket::close()
     int shutdownRet = ::shutdown(myFD, SD_BOTH);
     if (shutdownRet != 0) {
       int error = WSAGetLastError();
-      ArLog::log(ArLog::Normal, "Shutdown %s socket (0x%x) returns %i (error = %i)",
+      ArLog::log(ArLog::Normal, "Shutdown %s socket (0x%p) returns %i (error = %i)",
                  toString(myType),
                  myFD,
                  shutdownRet, 
@@ -559,7 +560,7 @@ AREXPORT bool ArSocket::close()
     int closeRet = closesocket(myFD);
     if (closeRet  != 0) {
       int error = WSAGetLastError();
-      ArLog::log(ArLog::Normal, "Close %s socket (0x%x) returns %i (error = %i)",
+      ArLog::log(ArLog::Normal, "Close %s socket (0x%p) returns %i (error = %i)",
                  toString(myType),
                  myHost.c_str(),
                  closeRet,
